@@ -23,6 +23,7 @@ import json
 import logging
 import os
 import platform
+import re
 import socket
 import subprocess
 import sys
@@ -379,7 +380,7 @@ _opts = [
 # Passwords, tokens, hashes
 _auth_opts = [
     cfg.StrOpt('undercloud_db_password',
-               help=('Password used for MySQL databases. '
+               help=('Password used for MySQL root user. '
                      'If left unset, one will be automatically generated.')
                ),
     cfg.StrOpt('undercloud_admin_token',
@@ -1128,7 +1129,7 @@ def _process_drivers_and_hardware_types(instack_env):
     # use one variable for them.
     mgmt_interfaces = {'fake', 'ipmitool'}
     # TODO(dtantsur): can we somehow avoid hardcoding hardware types here?
-    for hw_type in ('redfish', 'idrac', 'ilo', 'irmc'):
+    for hw_type in ('redfish', 'idrac', 'ilo', 'irmc', 'staging-ovirt'):
         if hw_type in enabled_hardware_types:
             mgmt_interfaces.add(hw_type)
     for (hw_type, iface) in [('cisco-ucs-managed', 'ucsm'),
@@ -1385,6 +1386,27 @@ def _run_yum_update(instack_env):
     LOG.info('yum-update completed successfully')
 
 
+def _get_ovs_interfaces():
+    interfaces = glob.glob('/etc/sysconfig/network-scripts/ifcfg-*')
+    pattern = "OVSIntPort"
+    ovs_interfaces = []
+    for interface in interfaces:
+        with open(interface, "r") as text:
+            for line in text:
+                if re.findall(pattern, line):
+                    # FIXME (holser). It might be better to get interface from
+                    # DEVICE rather than name of file.
+                    ovs_interfaces.append(interface.split('-')[-1])
+    return ovs_interfaces
+
+
+def _run_restore_ovs_interfaces(interfaces):
+    for interface in interfaces:
+        LOG.info('Running restart OVS interface %s', interface)
+        _run_command(['sudo', 'ifup', interface])
+        LOG.info('Restart OVS interface %s completed successfully', interface)
+
+
 def _run_orc(instack_env):
     args = ['sudo', 'os-refresh-config']
     LOG.info('Running os-refresh-config')
@@ -1561,19 +1583,51 @@ def _clean_os_collect_config():
 
 
 def _create_mistral_config_environment(instack_env, mistral):
-    # Store the snmpd password in a Mistral environment so it can be accessed
+    # Store all the required passwords from the Undercloud
+    # in a Mistral environment so they can be accessed
     # by the Mistral actions.
-    snmpd_password = instack_env["UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD"]
 
-    env_name = "tripleo.undercloud-config"
+    config_data = {
+        'undercloud_ceilometer_snmpd_password':
+            instack_env['UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD'],
+        'undercloud_db_password':
+            instack_env['UNDERCLOUD_DB_PASSWORD']
+    }
+    env_name = 'tripleo.undercloud-config'
     try:
-        mistral.environments.get(env_name)
+        env_data = mistral.environments.get(env_name).variables
     except ks_exceptions.NotFound:
+        # If the environment is not created, we need to
+        # create it with the information in config_data
         mistral.environments.create(
             name=env_name,
-            variables=json.dumps({
-                "undercloud_ceilometer_snmpd_password": snmpd_password
-            }))
+            description='Undercloud configuration parameters',
+            variables=json.dumps(config_data, sort_keys=True)
+        )
+        return
+
+    # If we are upgrading from an environment without
+    # variables defined in config_data, we need to update
+    # the environment variables.
+
+    for var, value in iter(config_data.items()):
+        if var in env_data:
+            if env_data[var] != config_data[var]:
+                # Value in config_data is different
+                # need to update
+                env_data[var] = value
+        else:
+            # The value in config_data
+            # is new, we need to add it
+            env_data[var] = value
+
+    # Here we update the current environment
+    # with the variables updated
+    mistral.environments.update(
+        name=env_name,
+        description='Undercloud configuration parameters',
+        variables=json.dumps(env_data, sort_keys=True)
+    )
 
 
 def _wait_for_mistral_execution(timeout_at, mistral, execution, message='',
@@ -1627,7 +1681,7 @@ def _get_session():
     return session.Session(auth=auth_plugin)
 
 
-def _run_validation_groups(groups=[], mistral_url='', timeout=360,
+def _run_validation_groups(groups=[], mistral_url='', timeout=540,
                            fail_on_error=False):
     sess = _get_session()
     mistral = mistralclient.client(mistral_url=mistral_url, session=sess)
@@ -1851,13 +1905,8 @@ def install(instack_root, upgrade=False):
         _validate_configuration()
         instack_env = _generate_environment(instack_root)
         _generate_init_data(instack_env)
+        ovs_interfaces = _get_ovs_interfaces()
         if upgrade:
-            # We didn't complete the M->N upgrades correctly with a
-            # `nova-manage db online_data_migrations` command before. This
-            # could cause the post-upgrade db sync to fail. Better be safe
-            # than sorry and run it before package upgrade.
-            _run_command(['sudo', '-E', '/usr/bin/nova-manage', 'db',
-                          'online_data_migrations'])
             # Even if we backport https://review.openstack.org/#/c/457478/
             # into stable branches of puppet-ironic, we still need a way
             # to handle existing deployments.
@@ -1869,10 +1918,17 @@ def install(instack_root, upgrade=False):
             _die_tuskar_die()
         if CONF.undercloud_update_packages:
             _run_yum_clean_all(instack_env)
+            if ovs_interfaces:
+                _run_restore_ovs_interfaces(ovs_interfaces)
             _run_yum_update(instack_env)
         _handle_upgrade_fact(upgrade)
         _run_instack(instack_env)
         _run_orc(instack_env)
+        # FIXME (holser). The RC of issue is in OVS flow restore. Once
+        # 'systemctl reload openvswitch' is fixed ovs port restoration can be
+        # removed.
+        if ovs_interfaces:
+            _run_restore_ovs_interfaces(ovs_interfaces)
         _post_config(instack_env, upgrade)
         _run_command(['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         if upgrade and CONF.enable_validations:  # Run post-upgrade validations
@@ -1902,6 +1958,12 @@ def pre_upgrade():
     _run_live_command(args, name='systemctl stop')
     LOG.info('Services stopped successfully')
 
+    # Ensure nova data migrations are complete before upgrading packages
+    LOG.info('Running Nova online data migration')
+    _run_command(['sudo', '-E', '/usr/bin/nova-manage', 'db',
+                  'online_data_migrations'])
+    LOG.info('Nova online data migration completed')
+
     args = ['sudo', 'yum', 'install', '-y', 'ansible-pacemaker']
     LOG.info('Installing Ansible Pacemaker module')
     _run_live_command(args, name='install ansible')
